/*
 * $Id: XMLWriter.java,v 1.2 2005-03-04 18:11:49 bbissett Exp $
 */

/*
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package com.sun.xml.rpc.streaming;

import javax.xml.namespace.QName;

import com.sun.xml.rpc.util.xml.CDATA;

/**
 * <p> The XMLWriter interface is used to write XML documents. </p>
 *
 * <p> Concrete XMLWriters can be created using a XMLWriterFactory. </p>
 *
 * @see XMLWriterFactory
 *
 * @author JAX-RPC Development Team
 */
public interface XMLWriter {

    /**
     * Write the start tag for an element.
     */
    public void startElement(QName name);

    /**
     * Write the start tag for an element.
     */
    public void startElement(QName name, String prefix);

    /**
     * Write the start tag for an element.
     */
    public void startElement(String localName);

    /**
     * Write the start tag for an element.
     */
    public void startElement(String localName, String uri);

    /**
     * Write the start tag for an element.
     */
    public void startElement(String localName, String uri, String prefix);

    /**
     * Write an attribute of the current element.
     */
    public void writeAttribute(QName name, String value);

    /**
     * Write an attribute of the current element.
     */
    public void writeAttribute(String localName, String value);

    /**
     * Write an attribute of the current element.
     */
    public void writeAttribute(String localName, String uri, String value);

    /**
     * Write an attribute (unquoted) of the current element.
     */
    public void writeAttributeUnquoted(QName name, String value);

    /**
     * Write an attribute (unquoted) of the current element.
     */
    public void writeAttributeUnquoted(String localName, String value);

    /**
     * Write an attribute (unquoted) of the current element.
     */
    public void writeAttributeUnquoted(
        String localName,
        String uri,
        String value);

    /**
     * Write a namespace declaration of the current element.
     */
    public void writeNamespaceDeclaration(String prefix, String uri);

    /**
     * Write a namespace declaration of the current element. The prefix name
     * will be generated by the PrefixFactory currently configured for
     * this writer.
     */
    public void writeNamespaceDeclaration(String uri);

    /**
     * Write character data within an element.
     */
    public void writeChars(String chars);

    /**
     * Write character data within an element.
     */
    public void writeChars(CDATA chars);

    /**
     * Write character data within an element, skipping quoting.
     */
    public void writeCharsUnquoted(String chars);

    /**
     * Write character data within an element, skipping quoting.
     */
    public void writeCharsUnquoted(char[] buf, int offset, int len);

    /**
     * Write a comment within an element.
     */
    public void writeComment(String comment);

    /**
     * Write the end tag for the current element.
     */
    public void endElement();

    /**
     * Return the prefix factory in use by this writer.
     */
    public PrefixFactory getPrefixFactory();

    /**
     * Set the prefix factory to be used by this writer.
     */
    public void setPrefixFactory(PrefixFactory factory);

    /**
     * Return the URI for a given prefix.
     *
     * If the prefix is undeclared, return null.
     */
    public String getURI(String prefix);

    /**
     * Return a prefix for the given URI.
     *
     * <p> If no prefix for the given URI is in scope, return null. </p>
     */
    public String getPrefix(String uri);

    /**
     * Flush the writer and its underlying stream.
     */
    public void flush();

    /**
     * Close the writer and its underlying stream.
     */
    public void close();
}
